/*
 * dsp.c: oss interface
 * Copyright (C) 2000 Fabio Checconi <fchecconi@libero.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include "dsp.h"

int	_dsp_setfmt		( int *ufmt );
int	_dsp_setchn		( int *uchn );
int	_dsp_setsrate		( int *urate );
int	_dsp_setbuffering	( int size, int num );
int	_dsp_init		( void );
int	_dsp_getobuffer		( audio_buf_info *info );
int	_dsp_getibuffer		( audio_buf_info *info );

_dsp_control	defaults = { "/dev/dsp", NULL, -1, 4096, DUPLEX_MODE, 
	0, 0, 0, U8, 1, 8000, 0, 0, 0, 0 };
_dsp_control	*dev = NULL;

int
dsp_open( _dsp_control *udev )
{
	audio_buf_info info;
	int flags;

	if( udev == NULL ) {
		dev = &defaults;
	} else {
		dev = udev;
		dev->fd = -1;
	}
	if( ( dev->fd = open( dev->dev_name, dev->o_mode | O_NONBLOCK ) ) == -1 ) {
		return -1;
	}
	if( ( flags = fcntl( dev->fd, F_GETFL ) ) == -1 )
		return -1;
	if( ( fcntl( dev->fd, F_SETFL, flags & ~O_NONBLOCK ) ) == -1 )
		return -1;

	if( _dsp_init() == -1 ) {
		dsp_close();
		return -1;
	}

	switch( dev->o_mode ) {
		case RECORD_MODE: if( _dsp_getibuffer( &info ) == -1 ) {
					dsp_close();
					return -1;
				}
				break;

		default: if( _dsp_getobuffer( &info ) == -1 ) {
					dsp_close();
					return -1;
				}
				break;
	}

	dev->frags = info.fragstotal;
	dev->fragsize = info.fragsize;
	if( dev->buf_size == 0 || dev->set_buf_parms == 1 )
		dev->buf_size = info.fragsize;

	return 0;
}

void
dsp_close( void )
{
	close( dev->fd );
	dev->fd = -1;
}

int
_dsp_setfmt( int *ufmt )
{
	if( ioctl( dev->fd, SNDCTL_DSP_SETFMT, ufmt ) == -1 )
		return -1;

	dev->fmt = *ufmt;

	return 0;
}

int
_dsp_setchn( int *uchn )
{
	if( ioctl( dev->fd, SNDCTL_DSP_CHANNELS, uchn ) == -1 )
		return -1;

	dev->chn = *uchn;

	return 0;
}

int
_dsp_setsrate( int *urate )
{
	if ( ioctl( dev->fd, SNDCTL_DSP_SPEED, urate ) == -1 )
		return -1;

	dev->srate = *urate;

	return 0;
}

int
_dsp_setbuffering( int size, int num )
{
	int arg = (num&0xffff) << 16, exp = 0;

	while( size > 1 ) {
		size >>= 1;
		exp++;
	}

	arg |= (exp&0xffff);
	if ( ioctl( dev->fd, SNDCTL_DSP_SETFRAGMENT, &arg ) == -1 )
		return -1;

	return 0;
}

int
_dsp_init( void )
{	
	if( _dsp_setfmt( &dev->fmt ) == -1 )
		return -1;
	if( _dsp_setchn( &dev->chn ) == -1 )
		return -1;
	if( _dsp_setsrate( &dev->srate ) == -1 )
		return -1;
	if( dev->set_buf_parms == 1 )
		if( _dsp_setbuffering( dev->fragsize, dev->frags ) == -1 )
			return -1;

	return 0;
}

int
dsp_playback( unsigned char *data, unsigned long size )
{
	int blk, nblocks = size / dev->buf_size,
		carrier = size % dev->buf_size;

	dev->ptr = data;
	for( blk = 0; blk <= nblocks; blk++ ) {
		if( blk != nblocks ) {
			if( write( dev->fd, dev->ptr, dev->buf_size ) == -1 )
				return -1;
		} else
			write( dev->fd, dev->ptr, carrier );
		dev->ptr = dev->ptr + dev->buf_size;
	}

	return 0;
}

int
dsp_record( unsigned char *data, unsigned long size )
{
	int blk, nblocks = size / dev->buf_size,
		carrier = size % dev->buf_size;

	dev->ptr = data;
	for( blk = 0; blk <= nblocks; blk++ ) {
		if( blk != nblocks ) {
			if( read( dev->fd, dev->ptr, dev->buf_size ) == -1 )
				return -1;
		} else
			read( dev->fd, dev->ptr, carrier );
		dev->ptr = dev->ptr + dev->buf_size;
	}

	return 0;
}

int
dsp_sync( void )
{
	if( ioctl( dev->fd, SNDCTL_DSP_SYNC, 0 ) == -1 )
		return -1;

	return 0;	
}

int
dsp_reset( void )
{
	if( ioctl( dev->fd, SNDCTL_DSP_RESET, 0 ) == -1 )
		return -1;

	return 0;	
}

int
_dsp_getobuffer( audio_buf_info *info )
{
	if( ioctl( dev->fd, SNDCTL_DSP_GETOSPACE, info ) == -1 )
		return -1;

	return 0;
}

int
_dsp_getibuffer( audio_buf_info *info )
{
	if( ioctl( dev->fd, SNDCTL_DSP_GETISPACE, info ) == -1 )
		return -1;

	return 0;
}

int
dsp_avail( void )
{
	audio_buf_info info;

	if( dev->o_mode==PLAYBACK_MODE || dev->o_mode==DUPLEX_MODE ) {
		if( _dsp_getobuffer( &info ) == -1 )
			return -1;

		dev->ofrags = info.fragments;
		dev->obytes = info.bytes;
	}

	if( dev->o_mode==RECORD_MODE || dev->o_mode==DUPLEX_MODE ) {
		if( _dsp_getibuffer( &info ) == -1 )
			return -1;

		dev->ifrags = info.fragments;
		dev->ibytes = info.bytes;
	}

	return 0;
}
