/*
 * globals.c: global functions
 * Copyright (C) 2000 Fabio Checconi <fchecconi@libero.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gnome.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "error_box.h"
#include "globals.h"

char _custom_error_msg[255];

extern GtkWidget *_error_box;
static GtkWidget *create_dummy_pixmap( GtkWidget *widget, gboolean gnome_pixmap);

void
error_msg( const char *msg, int type, int val )
{
	char *txt = NULL;
	char *def = "General Error";

	switch( type ) {
		case STD_ERRNO:
		txt = (char*) malloc( strlen( msg ) + strlen( strerror ( val ) ) + 1 );
			if( txt != NULL )
				sprintf( txt, "%s %s", msg, strerror( val ) );
			else
				txt = def;
			break;
		case CUSTOM:
			txt = (char*) malloc( strlen( msg ) + 256 );
			if ( txt != NULL ) {
				if ( val == 0 )
					sprintf( txt, "%s %s", msg, _custom_error_msg );
				else
					sprintf( txt, "%s", msg );
			} else
				txt = def;
			break;
		default:
			txt = def;
	}

	_error_box = create_error_box( txt );
	gtk_widget_show( _error_box );
	free( txt );
}

unsigned long
xmalloc( unsigned long size )
{
	unsigned long p = (unsigned long) malloc( size );
	if( p == 0 ) {
		error_msg( "Not enough memory", CUSTOM, 1 );
		exit( -1 );
	}
	
	return p;
}

GtkWidget*
lookup_widget( GtkWidget *widget, const gchar *widget_name )
{
	GtkWidget *parent, *found_widget;

	for(;;) {
		if( GTK_IS_MENU( widget ) )
			parent = gtk_menu_get_attach_widget( GTK_MENU( widget ) );
		else
			parent = widget->parent;

		if( parent == NULL )
			break;
		widget = parent;
	}

	found_widget = (GtkWidget*) gtk_object_get_data( GTK_OBJECT( widget ), widget_name );
	if( !found_widget )
		g_warning( "Widget not found: %s", widget_name );

	return found_widget;
}

static char *dummy_pixmap_xpm[] = {
	"1 1 1 1",
	"  c None",
	" ",
	" "
};

static GtkWidget*
create_dummy_pixmap( GtkWidget *widget, gboolean gnome_pixmap )
{
	GdkColormap *colormap;
	GdkPixmap *gdkpixmap;
	GdkBitmap *mask;
	GtkWidget *pixmap;

	if( gnome_pixmap ) {
		return gnome_pixmap_new_from_xpm_d( dummy_pixmap_xpm );
	}

	colormap = gtk_widget_get_colormap( widget );
	gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d( NULL, colormap,
		&mask, NULL, dummy_pixmap_xpm );
	if( gdkpixmap == NULL )
		g_error( "Couldn't create replacement pixmap." );
	pixmap = gtk_pixmap_new( gdkpixmap, mask );
	gdk_pixmap_unref( gdkpixmap );
	gdk_bitmap_unref( mask );

	return pixmap;
}

GtkWidget*
create_pixmap( GtkWidget *widget, const gchar *filename,gboolean gnome_pixmap )
{
	GtkWidget *pixmap;
	GdkColormap *colormap;
	GdkPixmap *gdkpixmap;
	GdkBitmap *mask;
	gchar *pathname;

	pathname = gnome_pixmap_file( filename );
	if( !pathname ) {
		g_warning( "Couldn't find pixmap file: %s", filename );
		return create_dummy_pixmap( widget, gnome_pixmap );
	}

	if( gnome_pixmap ) {
		pixmap = gnome_pixmap_new_from_file( pathname );
		g_free( pathname );
		return pixmap;
	}

	colormap = gtk_widget_get_colormap( widget );
	gdkpixmap = gdk_pixmap_colormap_create_from_xpm( NULL, colormap, 
		&mask, NULL, pathname );
	if( gdkpixmap == NULL ) {
		g_warning( "Couldn't create pixmap from file: %s", pathname );
		g_free( pathname );
		return create_dummy_pixmap( widget, gnome_pixmap );
	}
	g_free( pathname );

	pixmap = gtk_pixmap_new( gdkpixmap, mask );
	gdk_pixmap_unref( gdkpixmap );
	gdk_bitmap_unref( mask );

	return pixmap;
}

GdkImlibImage*
create_image( const gchar *filename )
{
	GdkImlibImage *image;
	gchar *pathname;

	pathname = gnome_pixmap_file( filename );
	if( !pathname ) {
		g_warning( "Couldn't find pixmap file: %s", filename );
		return NULL;
	}

	image = gdk_imlib_load_image( pathname );
	g_free( pathname );

	return image;
}

